/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/. */








    
        
    
    /// https://drafts.csswg.org/css-color/#color
    pub mod color {
        #[allow(unused_imports)]
        use cssparser::{Parser, BasicParseError, Token};
        #[allow(unused_imports)]
        use crate::parser::{Parse, ParserContext};
        #[allow(unused_imports)]
        use crate::properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use crate::values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use crate::error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use crate::properties::longhands;
        #[allow(unused_imports)]
        use crate::properties::{LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use crate::properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use crate::properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseErrorKind;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseErrorKind};
        #[allow(unused_imports)]
        use crate::values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use crate::values::{computed, generics, specified};
        #[allow(unused_imports)]
        use crate::Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use crate::values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use crate::values::specified::ColorPropertyValue as SpecifiedValue;
        pub mod computed_value {
            pub use crate::values::computed::ColorPropertyValue as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { ::cssparser::RGBA::new(0, 0, 0, 255) }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(
            context: &ParserContext,
            input: &mut Parser<'i, 't>,
        ) -> Result<SpecifiedValue, ParseError<'i>> {
            specified::ColorPropertyValue::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            context.for_non_inherited_property =
                    None;

            let specified_value = match *declaration {
                PropertyDeclaration::Color(ref value) => value,
                PropertyDeclaration::CSSWideKeyword(ref declaration) => {
                    debug_assert_eq!(declaration.id, LonghandId::Color);
                    match declaration.keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_color();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_color();
                        }
                    }
                    return;
                }
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };



                let computed = specified_value.to_computed_value(context);
                    context.builder.set_color(computed)
        }

        pub fn parse_declared<'i, 't>(
            context: &ParserContext,
            input: &mut Parser<'i, 't>,
        ) -> Result<PropertyDeclaration, ParseError<'i>> {
                parse(context, input)
                .map(PropertyDeclaration::Color)
        }
    }



// FIXME(#15973): Add servo support for system colors
//
// FIXME(emilio): Move outside of mako.
pub mod system_colors {
    
    use crate::gecko_bindings::bindings::Gecko_GetLookAndFeelSystemColor;
    use crate::gecko_bindings::structs::root::mozilla::LookAndFeel_ColorID;
    use std::fmt::{self, Write};
    use style_traits::{CssWriter, ToCss};
    use crate::values::computed::{Context, ToComputedValue};

    pub type SystemColor = LookAndFeel_ColorID;

    // It's hard to implement MallocSizeOf for LookAndFeel_ColorID because it
    // is a bindgen type. So we implement it on the typedef instead.
    malloc_size_of_is_0!(SystemColor);

    impl ToCss for SystemColor {
        fn to_css<W>(&self, dest: &mut CssWriter<W>) -> fmt::Result
        where
            W: Write,
        {
            let s = match *self {
                    LookAndFeel_ColorID::eColorID_activeborder => "activeborder",
                    LookAndFeel_ColorID::eColorID_activecaption => "activecaption",
                    LookAndFeel_ColorID::eColorID_appworkspace => "appworkspace",
                    LookAndFeel_ColorID::eColorID_background => "background",
                    LookAndFeel_ColorID::eColorID_buttonface => "buttonface",
                    LookAndFeel_ColorID::eColorID_buttonhighlight => "buttonhighlight",
                    LookAndFeel_ColorID::eColorID_buttonshadow => "buttonshadow",
                    LookAndFeel_ColorID::eColorID_buttontext => "buttontext",
                    LookAndFeel_ColorID::eColorID_captiontext => "captiontext",
                    LookAndFeel_ColorID::eColorID_graytext => "graytext",
                    LookAndFeel_ColorID::eColorID_highlight => "highlight",
                    LookAndFeel_ColorID::eColorID_highlighttext => "highlighttext",
                    LookAndFeel_ColorID::eColorID_inactiveborder => "inactiveborder",
                    LookAndFeel_ColorID::eColorID_inactivecaption => "inactivecaption",
                    LookAndFeel_ColorID::eColorID_inactivecaptiontext => "inactivecaptiontext",
                    LookAndFeel_ColorID::eColorID_infobackground => "infobackground",
                    LookAndFeel_ColorID::eColorID_infotext => "infotext",
                    LookAndFeel_ColorID::eColorID_menu => "menu",
                    LookAndFeel_ColorID::eColorID_menutext => "menutext",
                    LookAndFeel_ColorID::eColorID_scrollbar => "scrollbar",
                    LookAndFeel_ColorID::eColorID_threeddarkshadow => "threeddarkshadow",
                    LookAndFeel_ColorID::eColorID_threedface => "threedface",
                    LookAndFeel_ColorID::eColorID_threedhighlight => "threedhighlight",
                    LookAndFeel_ColorID::eColorID_threedlightshadow => "threedlightshadow",
                    LookAndFeel_ColorID::eColorID_threedshadow => "threedshadow",
                    LookAndFeel_ColorID::eColorID_window => "window",
                    LookAndFeel_ColorID::eColorID_windowframe => "windowframe",
                    LookAndFeel_ColorID::eColorID_windowtext => "windowtext",
                    LookAndFeel_ColorID::eColorID__moz_buttondefault => "-moz-buttondefault",
                    LookAndFeel_ColorID::eColorID__moz_buttonhoverface => "-moz-buttonhoverface",
                    LookAndFeel_ColorID::eColorID__moz_buttonhovertext => "-moz-buttonhovertext",
                    LookAndFeel_ColorID::eColorID__moz_cellhighlight => "-moz-cellhighlight",
                    LookAndFeel_ColorID::eColorID__moz_cellhighlighttext => "-moz-cellhighlighttext",
                    LookAndFeel_ColorID::eColorID__moz_eventreerow => "-moz-eventreerow",
                    LookAndFeel_ColorID::eColorID__moz_field => "-moz-field",
                    LookAndFeel_ColorID::eColorID__moz_fieldtext => "-moz-fieldtext",
                    LookAndFeel_ColorID::eColorID__moz_dialog => "-moz-dialog",
                    LookAndFeel_ColorID::eColorID__moz_dialogtext => "-moz-dialogtext",
                    LookAndFeel_ColorID::eColorID__moz_dragtargetzone => "-moz-dragtargetzone",
                    LookAndFeel_ColorID::eColorID__moz_gtk_info_bar_text => "-moz-gtk-info-bar-text",
                    LookAndFeel_ColorID::eColorID__moz_html_cellhighlight => "-moz-html-cellhighlight",
                    LookAndFeel_ColorID::eColorID__moz_html_cellhighlighttext => "-moz-html-cellhighlighttext",
                    LookAndFeel_ColorID::eColorID__moz_mac_buttonactivetext => "-moz-mac-buttonactivetext",
                    LookAndFeel_ColorID::eColorID__moz_mac_chrome_active => "-moz-mac-chrome-active",
                    LookAndFeel_ColorID::eColorID__moz_mac_chrome_inactive => "-moz-mac-chrome-inactive",
                    LookAndFeel_ColorID::eColorID__moz_mac_defaultbuttontext => "-moz-mac-defaultbuttontext",
                    LookAndFeel_ColorID::eColorID__moz_mac_focusring => "-moz-mac-focusring",
                    LookAndFeel_ColorID::eColorID__moz_mac_menuselect => "-moz-mac-menuselect",
                    LookAndFeel_ColorID::eColorID__moz_mac_menushadow => "-moz-mac-menushadow",
                    LookAndFeel_ColorID::eColorID__moz_mac_menutextdisable => "-moz-mac-menutextdisable",
                    LookAndFeel_ColorID::eColorID__moz_mac_menutextselect => "-moz-mac-menutextselect",
                    LookAndFeel_ColorID::eColorID__moz_mac_disabledtoolbartext => "-moz-mac-disabledtoolbartext",
                    LookAndFeel_ColorID::eColorID__moz_mac_secondaryhighlight => "-moz-mac-secondaryhighlight",
                    LookAndFeel_ColorID::eColorID__moz_mac_vibrancy_light => "-moz-mac-vibrancy-light",
                    LookAndFeel_ColorID::eColorID__moz_mac_vibrancy_dark => "-moz-mac-vibrancy-dark",
                    LookAndFeel_ColorID::eColorID__moz_mac_vibrant_titlebar_light => "-moz-mac-vibrant-titlebar-light",
                    LookAndFeel_ColorID::eColorID__moz_mac_vibrant_titlebar_dark => "-moz-mac-vibrant-titlebar-dark",
                    LookAndFeel_ColorID::eColorID__moz_mac_menupopup => "-moz-mac-menupopup",
                    LookAndFeel_ColorID::eColorID__moz_mac_menuitem => "-moz-mac-menuitem",
                    LookAndFeel_ColorID::eColorID__moz_mac_active_menuitem => "-moz-mac-active-menuitem",
                    LookAndFeel_ColorID::eColorID__moz_mac_source_list => "-moz-mac-source-list",
                    LookAndFeel_ColorID::eColorID__moz_mac_source_list_selection => "-moz-mac-source-list-selection",
                    LookAndFeel_ColorID::eColorID__moz_mac_active_source_list_selection => "-moz-mac-active-source-list-selection",
                    LookAndFeel_ColorID::eColorID__moz_mac_tooltip => "-moz-mac-tooltip",
                    LookAndFeel_ColorID::eColorID__moz_menuhover => "-moz-menuhover",
                    LookAndFeel_ColorID::eColorID__moz_menuhovertext => "-moz-menuhovertext",
                    LookAndFeel_ColorID::eColorID__moz_menubartext => "-moz-menubartext",
                    LookAndFeel_ColorID::eColorID__moz_menubarhovertext => "-moz-menubarhovertext",
                    LookAndFeel_ColorID::eColorID__moz_oddtreerow => "-moz-oddtreerow",
                    LookAndFeel_ColorID::eColorID__moz_win_mediatext => "-moz-win-mediatext",
                    LookAndFeel_ColorID::eColorID__moz_win_communicationstext => "-moz-win-communicationstext",
                    LookAndFeel_ColorID::eColorID__moz_win_accentcolor => "-moz-win-accentcolor",
                    LookAndFeel_ColorID::eColorID__moz_win_accentcolortext => "-moz-win-accentcolortext",
                    LookAndFeel_ColorID::eColorID__moz_nativehyperlinktext => "-moz-nativehyperlinktext",
                    LookAndFeel_ColorID::eColorID__moz_comboboxtext => "-moz-comboboxtext",
                    LookAndFeel_ColorID::eColorID__moz_combobox => "-moz-combobox",
                    LookAndFeel_ColorID::eColorID_WindowBackground => "WindowBackground",
                    LookAndFeel_ColorID::eColorID_WindowForeground => "WindowForeground",
                    LookAndFeel_ColorID::eColorID_WidgetBackground => "WidgetBackground",
                    LookAndFeel_ColorID::eColorID_WidgetForeground => "WidgetForeground",
                    LookAndFeel_ColorID::eColorID_WidgetSelectBackground => "WidgetSelectBackground",
                    LookAndFeel_ColorID::eColorID_WidgetSelectForeground => "WidgetSelectForeground",
                    LookAndFeel_ColorID::eColorID_Widget3DHighlight => "Widget3DHighlight",
                    LookAndFeel_ColorID::eColorID_Widget3DShadow => "Widget3DShadow",
                    LookAndFeel_ColorID::eColorID_TextBackground => "TextBackground",
                    LookAndFeel_ColorID::eColorID_TextForeground => "TextForeground",
                    LookAndFeel_ColorID::eColorID_TextSelectBackground => "TextSelectBackground",
                    LookAndFeel_ColorID::eColorID_TextSelectForeground => "TextSelectForeground",
                    LookAndFeel_ColorID::eColorID_TextSelectForegroundCustom => "TextSelectForegroundCustom",
                    LookAndFeel_ColorID::eColorID_TextSelectBackgroundDisabled => "TextSelectBackgroundDisabled",
                    LookAndFeel_ColorID::eColorID_TextSelectBackgroundAttention => "TextSelectBackgroundAttention",
                    LookAndFeel_ColorID::eColorID_TextHighlightBackground => "TextHighlightBackground",
                    LookAndFeel_ColorID::eColorID_TextHighlightForeground => "TextHighlightForeground",
                    LookAndFeel_ColorID::eColorID_IMERawInputBackground => "IMERawInputBackground",
                    LookAndFeel_ColorID::eColorID_IMERawInputForeground => "IMERawInputForeground",
                    LookAndFeel_ColorID::eColorID_IMERawInputUnderline => "IMERawInputUnderline",
                    LookAndFeel_ColorID::eColorID_IMESelectedRawTextBackground => "IMESelectedRawTextBackground",
                    LookAndFeel_ColorID::eColorID_IMESelectedRawTextForeground => "IMESelectedRawTextForeground",
                    LookAndFeel_ColorID::eColorID_IMESelectedRawTextUnderline => "IMESelectedRawTextUnderline",
                    LookAndFeel_ColorID::eColorID_IMEConvertedTextBackground => "IMEConvertedTextBackground",
                    LookAndFeel_ColorID::eColorID_IMEConvertedTextForeground => "IMEConvertedTextForeground",
                    LookAndFeel_ColorID::eColorID_IMEConvertedTextUnderline => "IMEConvertedTextUnderline",
                    LookAndFeel_ColorID::eColorID_IMESelectedConvertedTextBackground => "IMESelectedConvertedTextBackground",
                    LookAndFeel_ColorID::eColorID_IMESelectedConvertedTextForeground => "IMESelectedConvertedTextForeground",
                    LookAndFeel_ColorID::eColorID_IMESelectedConvertedTextUnderline => "IMESelectedConvertedTextUnderline",
                    LookAndFeel_ColorID::eColorID_SpellCheckerUnderline => "SpellCheckerUnderline",
                LookAndFeel_ColorID::eColorID_LAST_COLOR => unreachable!(),
            };
            dest.write_str(s)
        }
    }

    impl ToComputedValue for SystemColor {
        type ComputedValue = u32; // nscolor

        #[inline]
        fn to_computed_value(&self, cx: &Context) -> Self::ComputedValue {
            unsafe {
                Gecko_GetLookAndFeelSystemColor(
                    *self as i32,
                    cx.device().pres_context(),
                )
            }
        }

        #[inline]
        fn from_computed_value(_: &Self::ComputedValue) -> Self {
            unreachable!()
        }
    }

    impl SystemColor {
        pub fn from_ident<'i, 't>(ident: &str) -> Result<Self, ()> {
            ascii_case_insensitive_phf_map! {
                color_name -> SystemColor = {
                        "activeborder" => LookAndFeel_ColorID::eColorID_activeborder,
                        "activecaption" => LookAndFeel_ColorID::eColorID_activecaption,
                        "appworkspace" => LookAndFeel_ColorID::eColorID_appworkspace,
                        "background" => LookAndFeel_ColorID::eColorID_background,
                        "buttonface" => LookAndFeel_ColorID::eColorID_buttonface,
                        "buttonhighlight" => LookAndFeel_ColorID::eColorID_buttonhighlight,
                        "buttonshadow" => LookAndFeel_ColorID::eColorID_buttonshadow,
                        "buttontext" => LookAndFeel_ColorID::eColorID_buttontext,
                        "captiontext" => LookAndFeel_ColorID::eColorID_captiontext,
                        "graytext" => LookAndFeel_ColorID::eColorID_graytext,
                        "highlight" => LookAndFeel_ColorID::eColorID_highlight,
                        "highlighttext" => LookAndFeel_ColorID::eColorID_highlighttext,
                        "inactiveborder" => LookAndFeel_ColorID::eColorID_inactiveborder,
                        "inactivecaption" => LookAndFeel_ColorID::eColorID_inactivecaption,
                        "inactivecaptiontext" => LookAndFeel_ColorID::eColorID_inactivecaptiontext,
                        "infobackground" => LookAndFeel_ColorID::eColorID_infobackground,
                        "infotext" => LookAndFeel_ColorID::eColorID_infotext,
                        "menu" => LookAndFeel_ColorID::eColorID_menu,
                        "menutext" => LookAndFeel_ColorID::eColorID_menutext,
                        "scrollbar" => LookAndFeel_ColorID::eColorID_scrollbar,
                        "threeddarkshadow" => LookAndFeel_ColorID::eColorID_threeddarkshadow,
                        "threedface" => LookAndFeel_ColorID::eColorID_threedface,
                        "threedhighlight" => LookAndFeel_ColorID::eColorID_threedhighlight,
                        "threedlightshadow" => LookAndFeel_ColorID::eColorID_threedlightshadow,
                        "threedshadow" => LookAndFeel_ColorID::eColorID_threedshadow,
                        "window" => LookAndFeel_ColorID::eColorID_window,
                        "windowframe" => LookAndFeel_ColorID::eColorID_windowframe,
                        "windowtext" => LookAndFeel_ColorID::eColorID_windowtext,
                        "-moz-buttondefault" => LookAndFeel_ColorID::eColorID__moz_buttondefault,
                        "-moz-buttonhoverface" => LookAndFeel_ColorID::eColorID__moz_buttonhoverface,
                        "-moz-buttonhovertext" => LookAndFeel_ColorID::eColorID__moz_buttonhovertext,
                        "-moz-cellhighlight" => LookAndFeel_ColorID::eColorID__moz_cellhighlight,
                        "-moz-cellhighlighttext" => LookAndFeel_ColorID::eColorID__moz_cellhighlighttext,
                        "-moz-eventreerow" => LookAndFeel_ColorID::eColorID__moz_eventreerow,
                        "-moz-field" => LookAndFeel_ColorID::eColorID__moz_field,
                        "-moz-fieldtext" => LookAndFeel_ColorID::eColorID__moz_fieldtext,
                        "-moz-dialog" => LookAndFeel_ColorID::eColorID__moz_dialog,
                        "-moz-dialogtext" => LookAndFeel_ColorID::eColorID__moz_dialogtext,
                        "-moz-dragtargetzone" => LookAndFeel_ColorID::eColorID__moz_dragtargetzone,
                        "-moz-gtk-info-bar-text" => LookAndFeel_ColorID::eColorID__moz_gtk_info_bar_text,
                        "-moz-html-cellhighlight" => LookAndFeel_ColorID::eColorID__moz_html_cellhighlight,
                        "-moz-html-cellhighlighttext" => LookAndFeel_ColorID::eColorID__moz_html_cellhighlighttext,
                        "-moz-mac-buttonactivetext" => LookAndFeel_ColorID::eColorID__moz_mac_buttonactivetext,
                        "-moz-mac-chrome-active" => LookAndFeel_ColorID::eColorID__moz_mac_chrome_active,
                        "-moz-mac-chrome-inactive" => LookAndFeel_ColorID::eColorID__moz_mac_chrome_inactive,
                        "-moz-mac-defaultbuttontext" => LookAndFeel_ColorID::eColorID__moz_mac_defaultbuttontext,
                        "-moz-mac-focusring" => LookAndFeel_ColorID::eColorID__moz_mac_focusring,
                        "-moz-mac-menuselect" => LookAndFeel_ColorID::eColorID__moz_mac_menuselect,
                        "-moz-mac-menushadow" => LookAndFeel_ColorID::eColorID__moz_mac_menushadow,
                        "-moz-mac-menutextdisable" => LookAndFeel_ColorID::eColorID__moz_mac_menutextdisable,
                        "-moz-mac-menutextselect" => LookAndFeel_ColorID::eColorID__moz_mac_menutextselect,
                        "-moz-mac-disabledtoolbartext" => LookAndFeel_ColorID::eColorID__moz_mac_disabledtoolbartext,
                        "-moz-mac-secondaryhighlight" => LookAndFeel_ColorID::eColorID__moz_mac_secondaryhighlight,
                        "-moz-mac-vibrancy-light" => LookAndFeel_ColorID::eColorID__moz_mac_vibrancy_light,
                        "-moz-mac-vibrancy-dark" => LookAndFeel_ColorID::eColorID__moz_mac_vibrancy_dark,
                        "-moz-mac-vibrant-titlebar-light" => LookAndFeel_ColorID::eColorID__moz_mac_vibrant_titlebar_light,
                        "-moz-mac-vibrant-titlebar-dark" => LookAndFeel_ColorID::eColorID__moz_mac_vibrant_titlebar_dark,
                        "-moz-mac-menupopup" => LookAndFeel_ColorID::eColorID__moz_mac_menupopup,
                        "-moz-mac-menuitem" => LookAndFeel_ColorID::eColorID__moz_mac_menuitem,
                        "-moz-mac-active-menuitem" => LookAndFeel_ColorID::eColorID__moz_mac_active_menuitem,
                        "-moz-mac-source-list" => LookAndFeel_ColorID::eColorID__moz_mac_source_list,
                        "-moz-mac-source-list-selection" => LookAndFeel_ColorID::eColorID__moz_mac_source_list_selection,
                        "-moz-mac-active-source-list-selection" => LookAndFeel_ColorID::eColorID__moz_mac_active_source_list_selection,
                        "-moz-mac-tooltip" => LookAndFeel_ColorID::eColorID__moz_mac_tooltip,
                        "-moz-menuhover" => LookAndFeel_ColorID::eColorID__moz_menuhover,
                        "-moz-menuhovertext" => LookAndFeel_ColorID::eColorID__moz_menuhovertext,
                        "-moz-menubartext" => LookAndFeel_ColorID::eColorID__moz_menubartext,
                        "-moz-menubarhovertext" => LookAndFeel_ColorID::eColorID__moz_menubarhovertext,
                        "-moz-oddtreerow" => LookAndFeel_ColorID::eColorID__moz_oddtreerow,
                        "-moz-win-mediatext" => LookAndFeel_ColorID::eColorID__moz_win_mediatext,
                        "-moz-win-communicationstext" => LookAndFeel_ColorID::eColorID__moz_win_communicationstext,
                        "-moz-win-accentcolor" => LookAndFeel_ColorID::eColorID__moz_win_accentcolor,
                        "-moz-win-accentcolortext" => LookAndFeel_ColorID::eColorID__moz_win_accentcolortext,
                        "-moz-nativehyperlinktext" => LookAndFeel_ColorID::eColorID__moz_nativehyperlinktext,
                        "-moz-comboboxtext" => LookAndFeel_ColorID::eColorID__moz_comboboxtext,
                        "-moz-combobox" => LookAndFeel_ColorID::eColorID__moz_combobox,
                }
            }

            color_name(ident).cloned().ok_or(())
        }
    }
}
